from __future__ import absolute_import, unicode_literals

from django.apps import AppConfig


class WagtailSitemapsAppConfig(AppConfig):
    name = 'wagtail.contrib.wagtailsitemaps'
    label = 'wagtailsitemaps'
    verbose_name = "Wagtail sitemaps"
