{% load staticfiles i18n %}
<!-- Wagtail user bar embed code -->
<div class="wagtail-userbar-reset">
    <div class="wagtail-userbar wagtail-userbar--{{ position|default:'bottom-right' }}" data-wagtail-userbar>
        <link rel="stylesheet" href="{% static 'wagtailadmin/css/userbar.css' %}" type="text/css" />
        <div class="wagtail-userbar-nav">
            <div class="wagtail-icon wagtail-icon-wagtail wagtail-userbar-trigger" data-wagtail-userbar-trigger>
                <span class="wagtail-userbar-help-text">{% trans 'Go to Wagtail admin interface' %}</span>
            </div>
            <div class='wagtail-userbar-items'>
                {% for item in items %}
                    {{ item|safe }}
                {% endfor %}
            </div>
        </div>
        <script src="{% static 'wagtailadmin/js/userbar.js' %}"></script>
    </div>
</div>
<!-- end Wagtail user bar embed code -->
