{% load i18n wagtailadmin_tags %}
{{ formset.management_form }}
<ul class="multiple" id="id_{{ formset.prefix }}-FORMS">
    {% for form in formset.forms %}
        {% include "wagtailsearchpromotions/includes/searchpromotion_form.html" with form=form only %}
    {% endfor %}
</ul>

<script type="text/django-form-template" id="id_{{ formset.prefix }}-EMPTY_FORM_TEMPLATE">
{% escapescript %}
{% include "wagtailsearchpromotions/includes/searchpromotion_form.html" with form=formset.empty_form only %}
{% endescapescript %}
</script>

<p class="add">
    <a class="button bicolor icon icon-plus" id="id_{{ formset.prefix }}-ADD" value="Add">{% trans "Add recommended page" %}</a>
</p>
