from __future__ import absolute_import, unicode_literals


class RemovedInWagtail114Warning(DeprecationWarning):
    pass


removed_in_next_version_warning = RemovedInWagtail114Warning


class RemovedInWagtail115Warning(PendingDeprecationWarning):
    pass
