{% load i18n %}

<script>
    window.dateTimePickerTranslations = {
        months: [
            '{% trans "January" %}',
            '{% trans "February" %}',
            '{% trans "March" %}',
            '{% trans "April" %}',
            '{% trans "May" %}',
            '{% trans "June" %}',
            '{% trans "July" %}',
            '{% trans "August" %}',
            '{% trans "September" %}',
            '{% trans "October" %}',
            '{% trans "November" %}',
            '{% trans "December" %}'
        ],
        dayOfWeek: [
            '{% trans "Sun" %}',
            '{% trans "Mon" %}',
            '{% trans "Tue" %}',
            '{% trans "Wed" %}',
            '{% trans "Thu" %}',
            '{% trans "Fri" %}',
            '{% trans "Sat" %}'
        ]
    }
</script>
