# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2016-12-20 10:57
from __future__ import unicode_literals

from django.db import migrations
import wagtail.tests.testapp.models
import wagtail.wagtailcore.blocks
import wagtail.wagtailcore.fields


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0012_panelsettings_tabbedsettings'),
    ]

    operations = [
        migrations.AlterField(
            model_name='streampage',
            name='body',
            field=wagtail.wagtailcore.fields.StreamField([('text', wagtail.wagtailcore.blocks.CharBlock()), ('rich_text', wagtail.wagtailcore.blocks.RichTextBlock()), ('image', wagtail.tests.testapp.models.ExtendedImageChooserBlock())]),
        ),
    ]
