<div class="stream-menu {% if state == 'closed' %}stream-menu-closed{% endif %}" id="{{ menu_id }}">
    <a href="#" class="toggle"><span>Insert block</span></a>
    <div class="stream-menu-inner">
        {% regroup child_blocks by meta.group as grouped_child_blocks %}
        {% for child_blocks in grouped_child_blocks %}
            {% if child_blocks.grouper %}<h3>{{child_blocks.grouper}}</h3>{% endif %}
            <ul>
                {% for child_block in child_blocks.list %}
                    <li><button type="button" class="button action-add-block-{{ child_block.name }} icon icon-{{ child_block.meta.icon }}"><span>{{ child_block.label }}</span></button></li>
                {% endfor %}
            </ul>
        {% endfor %}
    </div>
</div>
