from __future__ import absolute_import, unicode_literals

from django.apps import AppConfig


class WagtailRoutablePageTestsAppConfig(AppConfig):
    name = 'wagtail.tests.routablepage'
    label = 'routablepagetests'
    verbose_name = "Wagtail routable page tests"
