{% extends "wagtailadmin/base.html" %}
{% load wagtailadmin_tags %}
{% load i18n %}

{% block titletag %}{% blocktrans with page_type=content_type.model_class.get_verbose_name %}New {{ page_type }}{% endblocktrans %}{% endblock %}
{% block bodyclass %}page-editor create model-{{ content_type.model }}{% endblock %}

{% block content %}

    <header class="merged tab-merged">
        {% explorer_breadcrumb parent_page include_self=1 %}

        <div class="row row-flush">
            <div class="left col9 header-title">
                <h1 class="icon icon-doc-empty-inverse">
                {% trans 'New' %} <span>{{ content_type.model_class.get_verbose_name }}</span></h1>
            </div>
        </div>
    </header>

    <form id="page-edit-form" action="{% url 'wagtailadmin_pages:add' content_type.app_label content_type.model parent_page.id %}" method="POST" novalidate{% if form.is_multipart %} enctype="multipart/form-data"{% endif %}>
        {% csrf_token %}
        <input type="hidden" name="next" value="{{ next }}">
        {{ edit_handler.render_form_content }}

        {% page_permissions parent_page as parent_page_perms %}
        <footer>
            <ul>
                <li class="actions">
                    <div class="dropdown dropup dropdown-button match-width">
                        <button type="submit" class="button action-save button-longrunning" tabindex="3" data-clicked-text="{% trans 'Saving...' %}"><span class="icon icon-spinner"></span><em>{% trans 'Save draft' %}</em></button>
                        <div class="dropdown-toggle icon icon-arrow-up"></div>
                        <ul role="menu">
                            {% if parent_page_perms.can_publish_subpage %}
                                <li>
                                    <button type="submit" name="action-publish" value="action-publish" class="button button-longrunning" tabindex="3" data-clicked-text="{% trans 'Publishing...' %}" {% if page.locked %}disabled {% endif %}><span class="icon icon-spinner"></span><em>{% trans 'Publish' %}</em></button>
                                </li>
                            {% endif %}
                            <li><input type="submit" name="action-submit" value="{% trans 'Submit for moderation' %}" class="button" /></li>
                        </ul>
                    </div>
                </li>

                <li class="preview">
                    {% trans 'Preview' as preview_label %}
                    {% if preview_modes|length > 1 %}
                        <div class="dropdown dropup dropdown-button match-width">
                            {% include "wagtailadmin/pages/_preview_button_on_create.html" with label=preview_label icon=1 %}
                            <div class="dropdown-toggle icon icon-arrow-up"></div>
                            <ul role="menu">
                                {% for mode_name, mode_display_name in preview_modes %}
                                    <li>
                                        {% include "wagtailadmin/pages/_preview_button_on_create.html" with mode=mode_name label=mode_display_name %}
                                    </li>
                                {% endfor %}
                            </ul>
                        </div>
                    {% else %}
                        {% include "wagtailadmin/pages/_preview_button_on_create.html" with label=preview_label icon=1 %}
                    {% endif %}
                </li>
            </ul>
        </footer>
    </form>

{% endblock %}

{% block extra_css %}
    {{ block.super }}
    {% include "wagtailadmin/pages/_editor_css.html" %}
{% endblock %}
{% block extra_js %}
    {{ block.super }}
    {% include "wagtailadmin/pages/_editor_js.html" %}

    <script>
        $(function(){
            $('#page-edit-form .tab-content section.active input').first().focus();

            /* Make user confirm before leaving the editor if there are unsaved changes */
            {% trans "This page has unsaved changes." as confirmation_message %}
            enableDirtyFormCheck(
                '#page-edit-form',
                {
                    confirmationMessage: '{{ confirmation_message|escapejs }}',

                    {% if has_unsaved_changes %}
                        alwaysDirty: true,
                    {% endif %}
                }
            );
        });
    </script>
{% endblock %}
