{% extends "wagtailadmin/base.html" %}
{% load i18n %}
{% block titletag %}{{ view.page_title }}{% endblock %}

{% block content %}
    {% include "wagtailadmin/shared/header.html" with title=view.page_title subtitle=view.get_page_subtitle icon=view.header_icon %}

    <div class="nice-padding">
        <p>
            {% trans 'This collection cannot be deleted, because it is not empty. It contains:' %}
        </p>
        <ul>
            {% for item_type in collection_contents %}
                <li>
                    {% if item_type.url %}
                        <a href="{{ item_type.url }}">{{ item_type.count_text }}</a>
                    {% else %}
                        {{ item_type.count_text }}
                    {% endif %}
                </li>
            {% endfor %}
        </ul>
    </div>
{% endblock %}
