from __future__ import absolute_import, unicode_literals

from django.apps import AppConfig


class WagtailSearchPromotionsAppConfig(AppConfig):
    name = 'wagtail.contrib.wagtailsearchpromotions'
    label = 'wagtailsearchpromotions'
    verbose_name = "Wagtail search promotions"
