{% extends "wagtailadmin/admin_base.html" %}
{% load staticfiles i18n %}
{% block titletag %}{% trans "Set your new password" %}{% endblock %}
{% block bodyclass %}login{% endblock %}

{% block extra_css %}
    {{ block.super }}

    <link rel="stylesheet" href="{% static 'wagtailadmin/css/layouts/login.css' %}" type="text/css" />
{% endblock %}

{% block furniture %}
    <div class="content-wrapper">

        {% if form.errors %}
            <div class="messages">
                <ul>
                    <li class="error">{% trans "The passwords do not match. Please try again." %}</li>
                </ul>
            </div>
        {% endif %}

        <form method="post" novalidate>
            {% csrf_token %}
            <h1>{% trans "Set your new password" %}</h1>

            <ul class="fields">
                <li>
                  <div class="field">
                      {{ form.new_password1.label_tag }}
                      {{ form.new_password1 }}
                  </div>
                </li>
                <li>
                  <div class="field">
                      {{ form.new_password2.label_tag }}
                      {{ form.new_password2 }}
                  </div>
                </li>
                <li class="submit">
                    <input type="submit" value="{% trans 'Reset password' %}" class="button" />
                </li>
            </ul>
        </form>
    </div>
{% endblock %}
